# SatMAD: Satellite Mission Analysis and Design for Python
#
# Copyright (C) 2020 Egemen Imre
#
# Licensed under GNU GPL v3.0. See LICENSE.rst for more info.
"""
Ground and Earth Location classes. These are very similar to
:class:`astropy.coordinates.EarthLocation` class.

"""

# class GroundLocation():
#
# class EarthLocation(GroundLocation):
